==> cloud-3.2.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/SnapshotCreateForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Snapshot entity create form.
 *
 * @ingroup aws_cloud
 */
class SnapshotCreateForm extends AwsCloudContentForm {

  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Snapshot $entity */
    $form = parent::buildForm($form, $form_state);
    $this->ec2Service->setCloudContext($cloud_context);
    $entity = $this->entity;

    $weight = -50;

    $form['snapshot'] = [
      '#type' => 'details',
      '#title' => $this->t('@title', ['@title' => $entity->getEntityType()->getSingularLabel()]),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['snapshot']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#required'      => FALSE,
    ];

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $volume_id = $this->getRequest()->query->get('volume_id');

    $form['snapshot']['volume_id'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Volume ID'),
      '#options'       => $this->getVolumeOptions($cloud_context, $module_name),
      '#default_value' => $volume_id,
      '#required'      => TRUE,
    ];

    $form['snapshot']['description'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Description'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->getDescription(),
      '#required'      => FALSE,
    ];

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    return $form;
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state) {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $result = [];
    if (!empty($entity->getVolumeId())) {
      $result = $this->ec2Service->createSnapshot([
        'Name'    => $entity->getName(),
        'VolumeId'    => $entity->getVolumeId(),
        'Description' => $entity->getDescription(),
      ]);
    }

    $start_time = is_object($result['StartTime'])
      ? strtotime($result['StartTime']->__toString())
      : strtotime($result['StartTime']);

    if (!empty($result['SnapshotId'])
    && ($entity->setSnapshotId($result['SnapshotId']))
    && ($entity->setStatus($result['State']))
    && ($entity->setStarted($start_time))
    && ($entity->setEncrypted($result['Encrypted'] ?? NULL))
    && ($entity->save())) {

      if (empty($entity->getName())) {
        $entity->setName($result['SnapshotId']);
        $entity->save();
      }

      $this->updateNameAndCreatedByTags($entity, $entity->getSnapshotId());

      $this->processOperationStatus($entity, 'created');

      $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list",
        [
          'cloud_context' => $entity->getCloudContext(),
        ]);
      $this->clearCacheValues();
    }
    else {

      $this->processOperationErrorStatus($entity, 'created');
    }

  }

  /**
   * Helper function to get volume options.
   *
   * @param string $cloud_context
   *   Cloud context to use in the query.
   * @param string $module_name
   *   Module name.
   *
   * @return array
   *   Volume options.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getVolumeOptions($cloud_context, $module_name) {
    $options = [];
    $params = [
      'cloud_context' => $cloud_context,
    ];

    // Get cloud service provider name.
    $cloud_name = self::convertUnderscoreToWhitespace($module_name);

    if (!$this->currentUser->hasPermission("view any {$cloud_name} volume")) {
      $params['uid'] = $this->currentUser->id();
    }

    $volumes = $this->entityTypeManager
      ->getStorage("{$module_name}_volume")
      ->loadByProperties($params);

    foreach ($volumes ?: [] as $volume) {
      if ($volume->getName() !== $volume->getVolumeId()) {
        $options[$volume->getVolumeId()] = "{$volume->getName()} ({$volume->getVolumeId()})";
      }
      else {
        $options[$volume->getVolumeId()] = $volume->getVolumeId();
      }
    }
    return $options;
  }

}

==> cloud-4.0.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/SnapshotCreateForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Snapshot entity create form.
 *
 * @ingroup aws_cloud
 */
class SnapshotCreateForm extends AwsCloudContentForm {

  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Snapshot $entity */
    $form = parent::buildForm($form, $form_state);
    $this->ec2Service->setCloudContext($cloud_context);
    $entity = $this->entity;

    $weight = -50;

    $form['snapshot'] = [
      '#type' => 'details',
      '#title' => $this->t('@title', ['@title' => $entity->getEntityType()->getSingularLabel()]),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['snapshot']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#required'      => FALSE,
    ];

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $volume_id = $this->getRequest()->query->get('volume_id');

    $form['snapshot']['volume_id'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Volume ID'),
      '#options'       => $this->getVolumeOptions($cloud_context, $module_name),
      '#default_value' => $volume_id,
      '#required'      => TRUE,
    ];

    $form['snapshot']['description'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Description'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->getDescription(),
      '#required'      => FALSE,
    ];

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    return $form;
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $result = [];
    if (!empty($entity->getVolumeId())) {
      $result = $this->ec2Service->createSnapshot([
        'Name'    => $entity->getName(),
        'VolumeId'    => $entity->getVolumeId(),
        'Description' => $entity->getDescription(),
      ]);
    }

    $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list", ['cloud_context' => $entity->getCloudContext()]);

    if (empty($result)) {
      $this->processOperationErrorStatus($entity, 'created');
      return;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'created remotely');
      return;
    }

    $start_time = is_object($result['StartTime'])
      ? strtotime($result['StartTime']->__toString())
      : strtotime($result['StartTime']);

    if (empty($entity->getName())) {
      $entity->setName($result['SnapshotId']);
    }
    $entity->setSnapshotId($result['SnapshotId']);
    $entity->setStatus($result['State']);
    $entity->setStarted($start_time);
    $entity->setEncrypted($result['Encrypted'] ?? NULL);
    $entity->save();

    $this->updateNameAndCreatedByTags($entity, $entity->getSnapshotId());
    $this->processOperationStatus($entity, 'created');
    $this->clearCacheValues($entity->getCacheTags());
    $this->dispatchSaveEvent($entity);
  }

  /**
   * Helper function to get volume options.
   *
   * @param string $cloud_context
   *   Cloud context to use in the query.
   * @param string $module_name
   *   Module name.
   *
   * @return array
   *   Volume options.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getVolumeOptions($cloud_context, $module_name): array {
    $options = [];
    $params = [
      'cloud_context' => $cloud_context,
    ];

    // Get cloud service provider name.
    $cloud_name = self::convertUnderscoreToWhitespace($module_name);

    if (!$this->currentUser->hasPermission("view any {$cloud_name} volume")) {
      $params['uid'] = $this->currentUser->id();
    }

    $volumes = $this->entityTypeManager
      ->getStorage("{$module_name}_volume")
      ->loadByProperties($params);

    foreach ($volumes ?: [] as $volume) {
      if ($volume->getName() !== $volume->getVolumeId()) {
        $options[$volume->getVolumeId()] = "{$volume->getName()} ({$volume->getVolumeId()})";
      }
      else {
        $options[$volume->getVolumeId()] = $volume->getVolumeId();
      }
    }

    natcasesort($options);
    return $options;
  }

}
